#pragma once
#include <thread>
#include <utility>
#include <atomic>
#include <sstream>

namespace Engine {
    class ConcurrentWorker;
}

class Engine::ConcurrentWorker {
private:
    std::atomic<bool> work_complete{ false };
    std::thread worker_thread;
public:
    [[nodiscard]] bool Done();
public:
    ~ConcurrentWorker();

    template <typename Function, typename... Args>
    explicit ConcurrentWorker(Function&& func, Args&&... args) : worker_thread([this, func = std::forward<Function>(func),
            ...args = std::forward<Args>(args)]() mutable { func(std::forward<Args>(args) ...); work_complete = true; })
            { worker_thread.detach(); }
};