#pragma once

#include <vector>
#include <Engine/types/InstancedTexture.h>
#include <Engine/types/entity/Entity.h>
#include <Engine/types/entity/Hud.h>
#include <JGL/types/RenderTarget.h>
#include <Engine/types/ConcurrentWorker.h>
#include <rewindow/types/window.h>
#include <thread>

namespace Engine {
    class Camera;
    class Scene;
}

class Engine::Scene {
private:
    bool destroying = false;
    bool last_rendered_to_window = false;
    float render_target_frame_delta = 0;
protected:
    // 25ms.
    u8 tick_rate = 40;
    bool should_tick = true;
    ConcurrentWorker* update_worker;
    std::string name;
    Camera* active_camera = nullptr;
    Entity* entity_list = nullptr;
    std::vector<InstancedTexture*> instanced_textures{};
    std::vector<InstancedTexture*> instanced_alpha_masks{};
protected:
    virtual void Update();
    virtual void Render();
public:
    [[nodiscard]] Entity* GetEntityList() { return entity_list; }
    [[nodiscard]] size_t EntityCount() const;
    [[nodiscard]] std::string GetName() const;
    [[nodiscard]] Camera* GetActiveCamera() const;
    [[nodiscard]] InstancedTexture* GetInstancedTexture(const InstancedSprite* user);
    [[nodiscard]] InstancedTexture* GetInstancedAlphaMask(const InstancedSprite* user);
    [[nodiscard]] bool ShouldTick() const;
    [[nodiscard]] float TickDeltaTime() const;
    [[nodiscard]] float FrameDeltaTime() const;
public:
    void ShouldTick(bool state);
    void SetActiveCamera(Camera* camera) { active_camera = camera; };
    void DestroyInstancedTexture(const InstancedTexture* itx);
    void DestroyInstancedAlphaMask(const InstancedTexture* alpha_mask);
    virtual void Unload();
    virtual void Init() {};
    void Run();

    void UpdateLoop();
    void Render(JGL::RenderTarget* render_target);
    void Render(ReWindow::RWindow* window);
public:
    explicit Scene(const std::string& name, const u8 tick_rate = 40) : tick_rate(tick_rate), name(name), entity_list(new Entity()) {}
    virtual ~Scene();
};
