#pragma once

#include <Engine/types/entity/Entity.h>
#include <Engine/types/entity/mixins/Living.h>
#include <Engine/types/entity/mixins/Renderable.h>

namespace Engine {
    class BoxCollider;
    class Movable;
    class Track;
}

/* I would expect you to set up such that the direct children would be made to perfectly
 * follow the track. The children of those children would be offset by the parents
 * movement every frame. This way, by default, Things could move on a train on the track. - Redacted. */
class Engine::Track : public Entity, public Renderable, public Living {
private:
    std::vector<Vector2> nodes{};
    std::vector<unsigned int> child_node_position{};
    void TranslateChild(Entity* entity);
protected:
    Vector2 GetNextNode(const Movable* movable);
public:
    // Returns whether the track forms a perfect loop *the first node and last node are the same*
    [[nodiscard]] bool Loops() const { return !nodes.empty() && nodes.front() == nodes.back(); }
    /// @returns The nearest node to a given entity.
    Vector2 ClosestNodeTo(const Movable* movable) const;
    Vector2 ClosestNodeTo(const BoxCollider* box_collider) const;

    void Render() override;
    void Update() override;
};
