#pragma once

#include <Colors.hpp>
#include <Engine/types/entity/Entity.h>
#include <Engine/types/entity/mixins/Movable.h>
#include <Engine/types/entity/mixins/Renderable.h>
#include <JGL/types/Texture.h>
#include <J3ML/Geometry/AABB2D.hpp>

namespace Engine {
    class Sprite;
}

using JGL::Texture;
class Engine::Sprite : public Entity, public Renderable, public Movable {
private:
    Texture* texture = nullptr;
    Texture* alpha_mask = nullptr;
protected:
    Vector2 scale = {1, 1};
    // Local space, Where the face_angle rotation should be preformed about.
    Vector2 origin = {0, 0};
    Color4 base_color = Colors::White;
protected:
    [[nodiscard]] virtual Texture* GetTexture();
    [[nodiscard]] virtual Texture* GetAlphaMask();
    void SetAlphaMask(Texture* new_alpha_mask);
public:
    void Render() override;
public:
    ~Sprite() override;
    explicit Sprite(const Vector2& pos, unsigned int layer = 0, const float face_angle = 0.0f, const Color4& base_color = Colors::White, Texture* texture = nullptr,
                    Texture* alpha_mask = nullptr) : Renderable(layer), Movable(pos, face_angle), texture(texture), alpha_mask(alpha_mask), base_color(base_color) {}
};