#pragma once

#include <vector>
#include <functional>
#include <Engine/types/entity/Entity.h>
#include <Engine/types/entity/mixins/BoxCollider.h>
#include <Engine/types/entity/mixins/Movable.h>
#include <Engine/types/entity/mixins/Renderable.h>
#include <Engine/types/entity/mixins/Living.h>

namespace Engine {
    class Trigger;
    class DynamicTrigger;
}

// Inheribable trigger.
class Engine::Trigger : public Entity, public Living, public BoxCollider, public Movable, public Renderable {
protected:
    Vector2 size;
public:
    [[nodiscard]] AABB2D GetBounds() const override;
    void Render() override;
    void SetSize(const Vector2& new_size) { size = new_size; }
public:
    explicit Trigger(const Vector2& position, const Vector2& size, bool enabled = true) :
    size(size), Entity(), Living(enabled), BoxCollider(enabled), Movable(position), Renderable(0, false) {}

    ~Trigger() override = default;
};

class Engine::DynamicTrigger final : public Trigger {
private:
    void RunActions();
    std::function<bool()> condition = nullptr;
    std::vector<std::function<void()>> actions{};
public:
    void Update() final;
    void SetCondition(const std::function<bool()>& condition);
    void AddAction(const std::function<void()>& action);
    void ClearActions();
public:
    explicit DynamicTrigger(const Vector2& position, const Vector2& size, bool enabled = true) : Trigger(position, size, enabled) {};

    DynamicTrigger(const Vector2& position, const Vector2& size, bool enabled, const std::function<bool()>& condition, const std::vector<std::function<void()>>& actions)
    : condition(condition), actions(actions), Trigger(position, size, enabled) {}

    DynamicTrigger(const Vector2& position, const Vector2& size, bool enabled, const std::function<bool()>& condition, const std::function<void()>& action) :
    Trigger(position, size, enabled) , condition(condition) { actions.push_back(action); }

    ~DynamicTrigger() final = default;
};