#include <Engine/types/Track.h>
#include <Engine/types/entity/mixins/Movable.h>
#include <Engine/types/entity/mixins/BoxCollider.h>
#include <JGL/JGL.h>

using namespace Engine;
void Track::Render() {
    if (!nodes.empty())
        J2D::DrawLines(Colors::Purples::DarkViolet, nodes.data(), nodes.size());
}

Vector2 Track::ClosestNodeTo(const Movable* movable) const {
    if (nodes.empty())
        throw std::runtime_error("Getting the closest node to a given movable but there is no nodes?");

    Vector2 result;
    float minimum = std::numeric_limits<float>::max();
    for (const auto& n : nodes) {
        float distance_sq = Vector2::DistanceSq(n, movable->GetPosition());
        if (distance_sq < minimum)
            minimum = distance_sq, result = n;
    }
    return result;
}

Vector2 Track::ClosestNodeTo(const BoxCollider* box_collider) const {
    if (nodes.empty())
        throw std::runtime_error("Getting the closest node to a given box_collider but there is no nodes?");

    Vector2 result;
    float minimum = std::numeric_limits<float>::max();
    auto aabb_minimum = box_collider->GetBounds().minPoint;
    auto aabb_maximum = box_collider->GetBounds().maxPoint;

    for (const auto& n : nodes) {
        Vector2 closest_aabb_point_to_current_node = { std::max(aabb_minimum.x, std::min(n.x, aabb_maximum.x)), std::max(aabb_minimum.y, std::min(n.y, aabb_maximum.y)) };
        float distance_squared = Vector2::DistanceSq(n, closest_aabb_point_to_current_node);

        if (distance_squared < minimum)
            minimum = distance_squared, result = n;
    }
    return result;
}

void Track::Update() {

}

void Track::TranslateChild(Entity* entity) {

    /*
    if (auto* m = dynamic_cast<Movable*>(entity))
        m->Move()
    auto m_pos = movable->GetPosition();
     */
}

Vector2 Track::GetNextNode(const Movable* movable) {
    auto m_pos = movable->GetPosition();


}
