#include <J3ML/J3ML.hpp>
#include <Engine/types/entity/Entity.h>

void Engine::Entity::AppendChild(Entity* entity) {
    if (entity->parent)
        entity->parent->RemoveChild(entity);

    entity->parent = this;
    children.push_back(entity);
}

Engine::Entity::~Entity() {
    for (auto* e : children)
        delete e;
}

void Engine::Entity::UpdateChildren() {
    for (auto& e : children) {
        if (!e)
            continue;

        e->Update();

        if (!e->children.empty())
            e->UpdateChildren();
    }
}

void Engine::Entity::DestroyChild(Entity* entity) {
    auto it = std::find(children.begin(), children.end(), entity);
    if (it != children.end())
        delete *it, children.erase(it);
}

void Engine::Entity::RemoveChild(Entity* entity) {
    auto it = std::find(children.begin(), children.end(), entity);
    if (it != children.end())
        children.erase(it);
    entity->parent = nullptr;
}

long Engine::Entity::GetCreationTime() const {
    return creation_time;
}

long Engine::Entity::GetAge() const {
    return std::chrono::duration_cast<std::chrono::microseconds>(std::chrono::high_resolution_clock::now().time_since_epoch()).count() - creation_time;
}

Engine::Entity* Engine::Entity::GetParent() const {
    return parent;
}

std::vector<Engine::Entity*> Engine::Entity::GetChildren() const {
    return children;
}

std::vector<Engine::Entity*> Engine::Entity::GetAllDescendents(const Engine::Entity* entity) {
    std::vector<Engine::Entity*> result = entity->GetChildren();

    if (result.empty())
        return result;

    for (auto* c : entity->GetChildren()) {
        auto descendants = Entity::GetAllDescendents(c);
        result.insert(result.end(), descendants.begin(), descendants.end());
    }
    return result;
}
