#include <Engine/types/entity/mixins/Movable.h>
#include <Engine/Globals.h>

using namespace J3ML;
void Engine::Movable::MoveX(float speed) {
    position.x = position.x + (speed * Globals::CurrentScene->TickDeltaTime());
}

void Engine::Movable::MoveY(float speed) {
    position.y = position.y + (speed * Globals::CurrentScene->TickDeltaTime());
}

void Engine::Movable::Move(float angle_rad, float speed) {
    position.x = position.x + (speed * Globals::CurrentScene->TickDeltaTime()) * Math::Cos(angle_rad);
    position.y = position.y + (speed * Globals::CurrentScene->TickDeltaTime()) * Math::Sin(angle_rad);
}

void Engine::Movable::MoveForward(float speed) {
    Move(face_angle, speed);
}

void Engine::Movable::MoveBackward(float speed) {
    speed = -speed;
    position.x = position.x + (speed * Globals::CurrentScene->TickDeltaTime()) * Math::Cos(face_angle);
    position.y = position.y + (speed * Globals::CurrentScene->TickDeltaTime()) * Math::Sin(face_angle);
}

void Engine::Movable::MoveLeft(float speed) {
    position.x = position.x + (speed * Globals::CurrentScene->TickDeltaTime()) * -Math::Sin(face_angle);
    position.y = position.y + (speed * Globals::CurrentScene->TickDeltaTime()) * Math::Cos(face_angle);
}

void Engine::Movable::MoveRight(float speed) {
    position.x = position.x + (speed * Globals::CurrentScene->TickDeltaTime()) * Math::Sin(face_angle);
    position.y = position.y + (speed * Globals::CurrentScene->TickDeltaTime()) * -Math::Cos(face_angle);
}

void Engine::Movable::SetRotation(float new_face_angle) {
    face_angle = new_face_angle;

    if (face_angle < 0)
        face_angle = fmod(face_angle + Math::Pi * 2, Math::Pi * 2);
    else if (face_angle >= 2 * M_PI)
        face_angle = fmod(face_angle, Math::Pi * 2);
}

void Engine::Movable::Rotate(float speed) {
    SetRotation(face_angle + speed * Globals::CurrentScene->TickDeltaTime());
}

float Engine::Movable::GetRotation() const {
    return face_angle;
}

Vector2 Engine::Movable::GetPosition() const {
    return position;
}