#include <Engine/types/entity/Trigger.h>
#include <JGL/JGL.h>

using namespace Engine;

void Trigger::Render() {
    if (ShouldRender()) {
        auto size = GetBounds().maxPoint - GetBounds().minPoint;
        J2D::OutlineRect(Colors::Red, position, size);
    }
}

AABB2D Trigger::GetBounds() const {
    return { position, position + size };
}


void DynamicTrigger::AddAction(const std::function<void()>& action) {
    actions.push_back(action);
}

void DynamicTrigger::Update() {
    bool success = false;
    if (condition)
        success = condition();

    if (success)
        RunActions();
}

void DynamicTrigger::RunActions() {
    for (const auto& a : actions)
        a();
}

void DynamicTrigger::ClearActions() {
    actions = {};
}

void DynamicTrigger::SetCondition(const std::function<bool()> & c) {
    condition = c;
}
